package com.winhc.fast.query.controller;

import com.winhc.fast.query.dao.HbaseRepository;
import com.winhc.fast.query.handler.HbaseResultHandler;
import com.winhc.fast.query.vo.HbaseQueryParamVo;
import com.winhc.fast.query.vo.ResponseVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.hadoop.hbase.client.Get;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/9/1 17:44
 * @Description:
 */
@Slf4j
@AllArgsConstructor
@RestController
@Api(tags = "hbase查询", value = "hbase")
@RequestMapping("hbase")
public class HbaseQueryController {
    private static final byte[] f = "F".getBytes();
    private HbaseRepository hbaseRepository;

    @ApiOperation(value = "scan前辍匹配，默认返回前100条数据")
    @GetMapping("scan/{tableName}/{rowPrefix}")
    public Mono<ResponseVo> scan(@PathVariable String tableName, @PathVariable String rowPrefix, @RequestParam(defaultValue = "100") Long size) {
        long start = System.currentTimeMillis();
        return hbaseRepository.scan(tableName, rowPrefix, size)
                .map(results -> ResponseVo.success(start, results.stream().map(HbaseResultHandler::transform).collect(Collectors.toList())))
                .defaultIfEmpty(ResponseVo.success(start, null))
                .onErrorResume(e -> Mono.just(ResponseVo.failure(start, e.getMessage())));
    }

    @ApiOperation(value = "点查")
    @GetMapping("get/{tableName}/{rowkey}")
    public Mono<ResponseVo> get(@PathVariable String tableName, @PathVariable String rowkey) {
        long start = System.currentTimeMillis();
        return hbaseRepository.get(tableName, rowkey)
                .map(r -> ResponseVo.success(start, HbaseResultHandler.transform(r)))
                .defaultIfEmpty(ResponseVo.success(start, null))
                .onErrorResume(e -> Mono.just(ResponseVo.failure(start, e.getMessage())));

    }

    @ApiOperation(value = "批量查询hbase，可指定字段")
    @PostMapping("bulk-get")
    public Object fastGetHbase(@RequestParam String tableName, @RequestBody HbaseQueryParamVo hbaseQueryParamVo) {
        long start = System.currentTimeMillis();
        if (hbaseQueryParamVo.getQueryKey() == null) {
            hbaseQueryParamVo.setQueryKey(new String[]{});
        }
        List<String> keys = Arrays.stream(hbaseQueryParamVo.getQueryKey())
                .filter(StringUtils::isNotEmpty)
                .map(String::toUpperCase)
                .collect(Collectors.toList());

        List<Get> gets = Arrays.stream(hbaseQueryParamVo.getQueryKey())
                .map(e -> new Get(e.getBytes(StandardCharsets.UTF_8)))
                .peek(get -> {
                    for (String key : keys) {
                        get.addColumn(f, key.getBytes());
                    }
                }).collect(Collectors.toList());
        return hbaseRepository.bulkGet(tableName, gets)
                .map(results -> ResponseVo.success(start, results.stream().map(HbaseResultHandler::transform).collect(Collectors.toList())))
                .defaultIfEmpty(ResponseVo.success(start, null))
                .onErrorResume(e -> Mono.just(ResponseVo.failure(start, e.getMessage())));
    }

}
