package com.winhc.fast.query.dao;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.client.*;
import org.apache.hadoop.hbase.filter.PageFilter;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author: XuJiakai
 * 2022/5/25 10:42
 */
@Slf4j
@Component
@AllArgsConstructor
public class HbaseRepository {
    private AsyncConnection connection;

    public Mono<Result> get(String tableName, String rowkey) {
        AsyncTable<AdvancedScanResultConsumer> table = connection.getTable(TableName.valueOf(tableName.toUpperCase()));
        Mono<Result> resultMono = Mono.fromFuture(table.get(new Get(rowkey.getBytes(StandardCharsets.UTF_8))));
        return resultMono;
    }

    public Mono<List<Result>> bulkGet(String tableName, List<Get> gets) {
        AsyncTable<AdvancedScanResultConsumer> table = connection.getTable(TableName.valueOf(tableName.toUpperCase()));
        return Mono.fromFuture(table.batchAll(gets));
    }

    public Mono<List<Result>> scan(String tableName, String rowkeyPrefix, Long size) {
        AsyncTable<AdvancedScanResultConsumer> table = connection.getTable(TableName.valueOf(tableName.toUpperCase()));
        Scan scan = new Scan();
        scan.setRowPrefixFilter(rowkeyPrefix.getBytes());
        PageFilter pageFilter = new PageFilter(size);
        scan.setFilter(pageFilter);
        return Mono.fromFuture(table.scanAll(scan));
    }
}
